# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import wireless_blocklist_list_params, wireless_blocklist_create_params, wireless_blocklist_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.wireless_blocklist_list_response import WirelessBlocklistListResponse
from ..types.wireless_blocklist_create_response import WirelessBlocklistCreateResponse
from ..types.wireless_blocklist_delete_response import WirelessBlocklistDeleteResponse
from ..types.wireless_blocklist_update_response import WirelessBlocklistUpdateResponse
from ..types.wireless_blocklist_retrieve_response import WirelessBlocklistRetrieveResponse

__all__ = ["WirelessBlocklistsResource", "AsyncWirelessBlocklistsResource"]


class WirelessBlocklistsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> WirelessBlocklistsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return WirelessBlocklistsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WirelessBlocklistsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return WirelessBlocklistsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        type: Literal["country", "mcc", "plmn"],
        values: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WirelessBlocklistCreateResponse:
        """
        Create a Wireless Blocklist to prevent SIMs from connecting to certain networks.

        Args:
          name: The name of the Wireless Blocklist.

          type: The type of wireless blocklist.

          values: Values to block. The values here depend on the `type` of Wireless Blocklist.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/wireless_blocklists",
            body=maybe_transform(
                {
                    "name": name,
                    "type": type,
                    "values": values,
                },
                wireless_blocklist_create_params.WirelessBlocklistCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WirelessBlocklistCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WirelessBlocklistRetrieveResponse:
        """
        Retrieve information about a Wireless Blocklist.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/wireless_blocklists/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WirelessBlocklistRetrieveResponse,
        )

    def update(
        self,
        *,
        name: str | Omit = omit,
        type: Literal["country", "mcc", "plmn"] | Omit = omit,
        values: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WirelessBlocklistUpdateResponse:
        """
        Update a Wireless Blocklist.

        Args:
          name: The name of the Wireless Blocklist.

          type: The type of wireless blocklist.

          values: Values to block. The values here depend on the `type` of Wireless Blocklist.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._patch(
            "/wireless_blocklists",
            body=maybe_transform(
                {
                    "name": name,
                    "type": type,
                    "values": values,
                },
                wireless_blocklist_update_params.WirelessBlocklistUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WirelessBlocklistUpdateResponse,
        )

    def list(
        self,
        *,
        filter_name: str | Omit = omit,
        filter_type: str | Omit = omit,
        filter_values: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WirelessBlocklistListResponse:
        """
        Get all Wireless Blocklists belonging to the user.

        Args:
          filter_name: The name of the Wireless Blocklist.

          filter_type: When the Private Wireless Gateway was last updated.

          filter_values: Values to filter on (inclusive).

          page_number: The page number to load.

          page_size: The size of the page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/wireless_blocklists",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_name": filter_name,
                        "filter_type": filter_type,
                        "filter_values": filter_values,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    wireless_blocklist_list_params.WirelessBlocklistListParams,
                ),
            ),
            cast_to=WirelessBlocklistListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WirelessBlocklistDeleteResponse:
        """
        Deletes the Wireless Blocklist.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/wireless_blocklists/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WirelessBlocklistDeleteResponse,
        )


class AsyncWirelessBlocklistsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncWirelessBlocklistsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncWirelessBlocklistsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWirelessBlocklistsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncWirelessBlocklistsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        type: Literal["country", "mcc", "plmn"],
        values: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WirelessBlocklistCreateResponse:
        """
        Create a Wireless Blocklist to prevent SIMs from connecting to certain networks.

        Args:
          name: The name of the Wireless Blocklist.

          type: The type of wireless blocklist.

          values: Values to block. The values here depend on the `type` of Wireless Blocklist.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/wireless_blocklists",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "type": type,
                    "values": values,
                },
                wireless_blocklist_create_params.WirelessBlocklistCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WirelessBlocklistCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WirelessBlocklistRetrieveResponse:
        """
        Retrieve information about a Wireless Blocklist.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/wireless_blocklists/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WirelessBlocklistRetrieveResponse,
        )

    async def update(
        self,
        *,
        name: str | Omit = omit,
        type: Literal["country", "mcc", "plmn"] | Omit = omit,
        values: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WirelessBlocklistUpdateResponse:
        """
        Update a Wireless Blocklist.

        Args:
          name: The name of the Wireless Blocklist.

          type: The type of wireless blocklist.

          values: Values to block. The values here depend on the `type` of Wireless Blocklist.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._patch(
            "/wireless_blocklists",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "type": type,
                    "values": values,
                },
                wireless_blocklist_update_params.WirelessBlocklistUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WirelessBlocklistUpdateResponse,
        )

    async def list(
        self,
        *,
        filter_name: str | Omit = omit,
        filter_type: str | Omit = omit,
        filter_values: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WirelessBlocklistListResponse:
        """
        Get all Wireless Blocklists belonging to the user.

        Args:
          filter_name: The name of the Wireless Blocklist.

          filter_type: When the Private Wireless Gateway was last updated.

          filter_values: Values to filter on (inclusive).

          page_number: The page number to load.

          page_size: The size of the page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/wireless_blocklists",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter_name": filter_name,
                        "filter_type": filter_type,
                        "filter_values": filter_values,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    wireless_blocklist_list_params.WirelessBlocklistListParams,
                ),
            ),
            cast_to=WirelessBlocklistListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WirelessBlocklistDeleteResponse:
        """
        Deletes the Wireless Blocklist.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/wireless_blocklists/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WirelessBlocklistDeleteResponse,
        )


class WirelessBlocklistsResourceWithRawResponse:
    def __init__(self, wireless_blocklists: WirelessBlocklistsResource) -> None:
        self._wireless_blocklists = wireless_blocklists

        self.create = to_raw_response_wrapper(
            wireless_blocklists.create,
        )
        self.retrieve = to_raw_response_wrapper(
            wireless_blocklists.retrieve,
        )
        self.update = to_raw_response_wrapper(
            wireless_blocklists.update,
        )
        self.list = to_raw_response_wrapper(
            wireless_blocklists.list,
        )
        self.delete = to_raw_response_wrapper(
            wireless_blocklists.delete,
        )


class AsyncWirelessBlocklistsResourceWithRawResponse:
    def __init__(self, wireless_blocklists: AsyncWirelessBlocklistsResource) -> None:
        self._wireless_blocklists = wireless_blocklists

        self.create = async_to_raw_response_wrapper(
            wireless_blocklists.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            wireless_blocklists.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            wireless_blocklists.update,
        )
        self.list = async_to_raw_response_wrapper(
            wireless_blocklists.list,
        )
        self.delete = async_to_raw_response_wrapper(
            wireless_blocklists.delete,
        )


class WirelessBlocklistsResourceWithStreamingResponse:
    def __init__(self, wireless_blocklists: WirelessBlocklistsResource) -> None:
        self._wireless_blocklists = wireless_blocklists

        self.create = to_streamed_response_wrapper(
            wireless_blocklists.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            wireless_blocklists.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            wireless_blocklists.update,
        )
        self.list = to_streamed_response_wrapper(
            wireless_blocklists.list,
        )
        self.delete = to_streamed_response_wrapper(
            wireless_blocklists.delete,
        )


class AsyncWirelessBlocklistsResourceWithStreamingResponse:
    def __init__(self, wireless_blocklists: AsyncWirelessBlocklistsResource) -> None:
        self._wireless_blocklists = wireless_blocklists

        self.create = async_to_streamed_response_wrapper(
            wireless_blocklists.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            wireless_blocklists.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            wireless_blocklists.update,
        )
        self.list = async_to_streamed_response_wrapper(
            wireless_blocklists.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            wireless_blocklists.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

from .._types import SequenceNotStr
from .shared_params.sim_card_status import SimCardStatus

__all__ = ["SimCardUpdateParams", "DataLimit"]


class SimCardUpdateParams(TypedDict, total=False):
    authorized_imeis: Optional[SequenceNotStr[str]]
    """List of IMEIs authorized to use a given SIM card."""

    data_limit: DataLimit
    """The SIM card individual data limit configuration."""

    sim_card_group_id: str
    """The group SIMCardGroup identification.

    This attribute can be <code>null</code> when it's present in an associated
    resource.
    """

    status: SimCardStatus

    tags: SequenceNotStr[str]
    """Searchable tags associated with the SIM card"""


class DataLimit(TypedDict, total=False):
    amount: str

    unit: Literal["MB", "GB"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["VersionConfigParam"]


class VersionConfigParam(TypedDict, total=False):
    percentage: Required[float]
    """Percentage of traffic for this version [1-99]"""

    version_id: Required[str]
    """Version ID string that references assistant_versions.version_id"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

from ..._types import SequenceNotStr

__all__ = ["ActionRequirementListParams", "Filter", "Page", "Sort"]


class ActionRequirementListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[id][in][], filter[requirement_type_id], filter[action_type],
    filter[status]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[size], page[number]
    """

    sort: Sort
    """Consolidated sort parameter (deepObject style). Originally: sort[value]"""


class Filter(TypedDict, total=False):
    id: SequenceNotStr[str]
    """Filter action requirements by a list of IDs"""

    action_type: Literal["au_id_verification"]
    """Filter action requirements by action type"""

    requirement_type_id: str
    """Filter action requirements by requirement type ID"""

    status: Literal["created", "pending", "completed", "cancelled", "failed"]
    """Filter action requirements by status"""


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""


class Sort(TypedDict, total=False):
    value: Literal["created_at", "-created_at", "updated_at", "-updated_at"]
    """Specifies the sort order for results.

    If not given, results are sorted by created_at in descending order.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ....._models import BaseModel
from .test_status import TestStatus

__all__ = ["TestRunResponse", "DetailStatus"]


class DetailStatus(BaseModel):
    name: str

    status: TestStatus
    """Represents the lifecycle of a test:

    - 'pending': Test is waiting to be executed.
    - 'starting': Test execution is initializing.
    - 'running': Test is currently executing.
    - 'passed': Test completed successfully.
    - 'failed': Test executed but did not pass.
    - 'error': An error occurred during test execution.
    """


class TestRunResponse(BaseModel):
    __test__ = False
    created_at: datetime
    """Timestamp when the test run was created and queued."""

    run_id: str
    """Unique identifier for this specific test run execution."""

    status: TestStatus
    """Represents the lifecycle of a test:

    - 'pending': Test is waiting to be executed.
    - 'starting': Test execution is initializing.
    - 'running': Test is currently executing.
    - 'passed': Test completed successfully.
    - 'failed': Test executed but did not pass.
    - 'error': An error occurred during test execution.
    """

    test_id: str
    """Identifier of the assistant test that was executed."""

    triggered_by: str
    """How this test run was initiated (manual, scheduled, or API)."""

    completed_at: Optional[datetime] = None
    """Timestamp when the test run finished execution."""

    conversation_id: Optional[str] = None
    """Identifier of the conversation created during test execution."""

    conversation_insights_id: Optional[str] = None
    """Identifier for conversation analysis and insights data."""

    detail_status: Optional[List[DetailStatus]] = None
    """Detailed evaluation results for each rubric criteria.

    Name is name of the criteria from the rubric and status is the result of the
    evaluation. This list will have a result for every criteria in the rubric
    section.
    """

    logs: Optional[str] = None
    """Detailed execution logs and debug information."""

    test_suite_run_id: Optional[str] = None
    """Identifier linking this run to a test suite execution batch."""

    updated_at: Optional[datetime] = None
    """Timestamp of the last update to this test run."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import public_internet_gateway_list_params, public_internet_gateway_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.public_internet_gateway_list_response import PublicInternetGatewayListResponse
from ..types.public_internet_gateway_create_response import PublicInternetGatewayCreateResponse
from ..types.public_internet_gateway_delete_response import PublicInternetGatewayDeleteResponse
from ..types.public_internet_gateway_retrieve_response import PublicInternetGatewayRetrieveResponse

__all__ = ["PublicInternetGatewaysResource", "AsyncPublicInternetGatewaysResource"]


class PublicInternetGatewaysResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PublicInternetGatewaysResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return PublicInternetGatewaysResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PublicInternetGatewaysResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return PublicInternetGatewaysResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str | Omit = omit,
        network_id: str | Omit = omit,
        region_code: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PublicInternetGatewayCreateResponse:
        """
        Create a new Public Internet Gateway.

        Args:
          name: A user specified name for the interface.

          network_id: The id of the network associated with the interface.

          region_code: The region the interface should be deployed to.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/public_internet_gateways",
            body=maybe_transform(
                {
                    "name": name,
                    "network_id": network_id,
                    "region_code": region_code,
                },
                public_internet_gateway_create_params.PublicInternetGatewayCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PublicInternetGatewayCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PublicInternetGatewayRetrieveResponse:
        """
        Retrieve a Public Internet Gateway.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/public_internet_gateways/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PublicInternetGatewayRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: public_internet_gateway_list_params.Filter | Omit = omit,
        page: public_internet_gateway_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PublicInternetGatewayListResponse:
        """
        List all Public Internet Gateways.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[network_id]

          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/public_internet_gateways",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                    },
                    public_internet_gateway_list_params.PublicInternetGatewayListParams,
                ),
            ),
            cast_to=PublicInternetGatewayListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PublicInternetGatewayDeleteResponse:
        """
        Delete a Public Internet Gateway.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/public_internet_gateways/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PublicInternetGatewayDeleteResponse,
        )


class AsyncPublicInternetGatewaysResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPublicInternetGatewaysResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPublicInternetGatewaysResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPublicInternetGatewaysResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncPublicInternetGatewaysResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str | Omit = omit,
        network_id: str | Omit = omit,
        region_code: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PublicInternetGatewayCreateResponse:
        """
        Create a new Public Internet Gateway.

        Args:
          name: A user specified name for the interface.

          network_id: The id of the network associated with the interface.

          region_code: The region the interface should be deployed to.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/public_internet_gateways",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "network_id": network_id,
                    "region_code": region_code,
                },
                public_internet_gateway_create_params.PublicInternetGatewayCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PublicInternetGatewayCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PublicInternetGatewayRetrieveResponse:
        """
        Retrieve a Public Internet Gateway.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/public_internet_gateways/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PublicInternetGatewayRetrieveResponse,
        )

    async def list(
        self,
        *,
        filter: public_internet_gateway_list_params.Filter | Omit = omit,
        page: public_internet_gateway_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PublicInternetGatewayListResponse:
        """
        List all Public Internet Gateways.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[network_id]

          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/public_internet_gateways",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                    },
                    public_internet_gateway_list_params.PublicInternetGatewayListParams,
                ),
            ),
            cast_to=PublicInternetGatewayListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PublicInternetGatewayDeleteResponse:
        """
        Delete a Public Internet Gateway.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/public_internet_gateways/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PublicInternetGatewayDeleteResponse,
        )


class PublicInternetGatewaysResourceWithRawResponse:
    def __init__(self, public_internet_gateways: PublicInternetGatewaysResource) -> None:
        self._public_internet_gateways = public_internet_gateways

        self.create = to_raw_response_wrapper(
            public_internet_gateways.create,
        )
        self.retrieve = to_raw_response_wrapper(
            public_internet_gateways.retrieve,
        )
        self.list = to_raw_response_wrapper(
            public_internet_gateways.list,
        )
        self.delete = to_raw_response_wrapper(
            public_internet_gateways.delete,
        )


class AsyncPublicInternetGatewaysResourceWithRawResponse:
    def __init__(self, public_internet_gateways: AsyncPublicInternetGatewaysResource) -> None:
        self._public_internet_gateways = public_internet_gateways

        self.create = async_to_raw_response_wrapper(
            public_internet_gateways.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            public_internet_gateways.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            public_internet_gateways.list,
        )
        self.delete = async_to_raw_response_wrapper(
            public_internet_gateways.delete,
        )


class PublicInternetGatewaysResourceWithStreamingResponse:
    def __init__(self, public_internet_gateways: PublicInternetGatewaysResource) -> None:
        self._public_internet_gateways = public_internet_gateways

        self.create = to_streamed_response_wrapper(
            public_internet_gateways.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            public_internet_gateways.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            public_internet_gateways.list,
        )
        self.delete = to_streamed_response_wrapper(
            public_internet_gateways.delete,
        )


class AsyncPublicInternetGatewaysResourceWithStreamingResponse:
    def __init__(self, public_internet_gateways: AsyncPublicInternetGatewaysResource) -> None:
        self._public_internet_gateways = public_internet_gateways

        self.create = async_to_streamed_response_wrapper(
            public_internet_gateways.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            public_internet_gateways.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            public_internet_gateways.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            public_internet_gateways.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ....._utils import PropertyInfo

__all__ = ["RecordingRecordingSidJsonParams"]


class RecordingRecordingSidJsonParams(TypedDict, total=False):
    account_sid: Required[str]

    call_sid: Required[str]

    status: Annotated[Literal["in-progress", "paused", "stopped"], PropertyInfo(alias="Status")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, cast

import httpx

from ..types import (
    phone_number_assignment_by_profile_assign_params,
    phone_number_assignment_by_profile_retrieve_phone_number_status_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.phone_number_assignment_by_profile_assign_response import PhoneNumberAssignmentByProfileAssignResponse
from ..types.phone_number_assignment_by_profile_retrieve_status_response import (
    PhoneNumberAssignmentByProfileRetrieveStatusResponse,
)
from ..types.phone_number_assignment_by_profile_retrieve_phone_number_status_response import (
    PhoneNumberAssignmentByProfileRetrievePhoneNumberStatusResponse,
)

__all__ = ["PhoneNumberAssignmentByProfileResource", "AsyncPhoneNumberAssignmentByProfileResource"]


class PhoneNumberAssignmentByProfileResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PhoneNumberAssignmentByProfileResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return PhoneNumberAssignmentByProfileResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PhoneNumberAssignmentByProfileResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return PhoneNumberAssignmentByProfileResourceWithStreamingResponse(self)

    def assign(
        self,
        *,
        messaging_profile_id: str,
        campaign_id: str | Omit = omit,
        tcr_campaign_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberAssignmentByProfileAssignResponse:
        """
        This endpoint allows you to link all phone numbers associated with a Messaging
        Profile to a campaign. **Please note:** if you want to assign phone numbers to a
        campaign that you did not create with Telnyx 10DLC services, this endpoint
        allows that provided that you've shared the campaign with Telnyx. In this case,
        only provide the parameter, `tcrCampaignId`, and not `campaignId`. In all other
        cases (where the campaign you're assigning was created with Telnyx 10DLC
        services), only provide `campaignId`, not `tcrCampaignId`.

        Args:
          messaging_profile_id: The ID of the messaging profile that you want to link to the specified campaign.

          campaign_id: The ID of the campaign you want to link to the specified messaging profile. If
              you supply this ID in the request, do not also include a tcrCampaignId.

          tcr_campaign_id: The TCR ID of the shared campaign you want to link to the specified messaging
              profile (for campaigns not created using Telnyx 10DLC services only). If you
              supply this ID in the request, do not also include a campaignId.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return cast(
            PhoneNumberAssignmentByProfileAssignResponse,
            self._post(
                "/phoneNumberAssignmentByProfile",
                body=maybe_transform(
                    {
                        "messaging_profile_id": messaging_profile_id,
                        "campaign_id": campaign_id,
                        "tcr_campaign_id": tcr_campaign_id,
                    },
                    phone_number_assignment_by_profile_assign_params.PhoneNumberAssignmentByProfileAssignParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, PhoneNumberAssignmentByProfileAssignResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def retrieve_phone_number_status(
        self,
        task_id: str,
        *,
        page: int | Omit = omit,
        records_per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberAssignmentByProfileRetrievePhoneNumberStatusResponse:
        """
        Check the status of the individual phone number/campaign assignments associated
        with the supplied `taskId`.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        return self._get(
            f"/phoneNumberAssignmentByProfile/{task_id}/phoneNumbers",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "records_per_page": records_per_page,
                    },
                    phone_number_assignment_by_profile_retrieve_phone_number_status_params.PhoneNumberAssignmentByProfileRetrievePhoneNumberStatusParams,
                ),
            ),
            cast_to=PhoneNumberAssignmentByProfileRetrievePhoneNumberStatusResponse,
        )

    def retrieve_status(
        self,
        task_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberAssignmentByProfileRetrieveStatusResponse:
        """
        Check the status of the task associated with assigning all phone numbers on a
        messaging profile to a campaign by `taskId`.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        return self._get(
            f"/phoneNumberAssignmentByProfile/{task_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberAssignmentByProfileRetrieveStatusResponse,
        )


class AsyncPhoneNumberAssignmentByProfileResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPhoneNumberAssignmentByProfileResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPhoneNumberAssignmentByProfileResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPhoneNumberAssignmentByProfileResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncPhoneNumberAssignmentByProfileResourceWithStreamingResponse(self)

    async def assign(
        self,
        *,
        messaging_profile_id: str,
        campaign_id: str | Omit = omit,
        tcr_campaign_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberAssignmentByProfileAssignResponse:
        """
        This endpoint allows you to link all phone numbers associated with a Messaging
        Profile to a campaign. **Please note:** if you want to assign phone numbers to a
        campaign that you did not create with Telnyx 10DLC services, this endpoint
        allows that provided that you've shared the campaign with Telnyx. In this case,
        only provide the parameter, `tcrCampaignId`, and not `campaignId`. In all other
        cases (where the campaign you're assigning was created with Telnyx 10DLC
        services), only provide `campaignId`, not `tcrCampaignId`.

        Args:
          messaging_profile_id: The ID of the messaging profile that you want to link to the specified campaign.

          campaign_id: The ID of the campaign you want to link to the specified messaging profile. If
              you supply this ID in the request, do not also include a tcrCampaignId.

          tcr_campaign_id: The TCR ID of the shared campaign you want to link to the specified messaging
              profile (for campaigns not created using Telnyx 10DLC services only). If you
              supply this ID in the request, do not also include a campaignId.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return cast(
            PhoneNumberAssignmentByProfileAssignResponse,
            await self._post(
                "/phoneNumberAssignmentByProfile",
                body=await async_maybe_transform(
                    {
                        "messaging_profile_id": messaging_profile_id,
                        "campaign_id": campaign_id,
                        "tcr_campaign_id": tcr_campaign_id,
                    },
                    phone_number_assignment_by_profile_assign_params.PhoneNumberAssignmentByProfileAssignParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, PhoneNumberAssignmentByProfileAssignResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def retrieve_phone_number_status(
        self,
        task_id: str,
        *,
        page: int | Omit = omit,
        records_per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberAssignmentByProfileRetrievePhoneNumberStatusResponse:
        """
        Check the status of the individual phone number/campaign assignments associated
        with the supplied `taskId`.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        return await self._get(
            f"/phoneNumberAssignmentByProfile/{task_id}/phoneNumbers",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "page": page,
                        "records_per_page": records_per_page,
                    },
                    phone_number_assignment_by_profile_retrieve_phone_number_status_params.PhoneNumberAssignmentByProfileRetrievePhoneNumberStatusParams,
                ),
            ),
            cast_to=PhoneNumberAssignmentByProfileRetrievePhoneNumberStatusResponse,
        )

    async def retrieve_status(
        self,
        task_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberAssignmentByProfileRetrieveStatusResponse:
        """
        Check the status of the task associated with assigning all phone numbers on a
        messaging profile to a campaign by `taskId`.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        return await self._get(
            f"/phoneNumberAssignmentByProfile/{task_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberAssignmentByProfileRetrieveStatusResponse,
        )


class PhoneNumberAssignmentByProfileResourceWithRawResponse:
    def __init__(self, phone_number_assignment_by_profile: PhoneNumberAssignmentByProfileResource) -> None:
        self._phone_number_assignment_by_profile = phone_number_assignment_by_profile

        self.assign = to_raw_response_wrapper(
            phone_number_assignment_by_profile.assign,
        )
        self.retrieve_phone_number_status = to_raw_response_wrapper(
            phone_number_assignment_by_profile.retrieve_phone_number_status,
        )
        self.retrieve_status = to_raw_response_wrapper(
            phone_number_assignment_by_profile.retrieve_status,
        )


class AsyncPhoneNumberAssignmentByProfileResourceWithRawResponse:
    def __init__(self, phone_number_assignment_by_profile: AsyncPhoneNumberAssignmentByProfileResource) -> None:
        self._phone_number_assignment_by_profile = phone_number_assignment_by_profile

        self.assign = async_to_raw_response_wrapper(
            phone_number_assignment_by_profile.assign,
        )
        self.retrieve_phone_number_status = async_to_raw_response_wrapper(
            phone_number_assignment_by_profile.retrieve_phone_number_status,
        )
        self.retrieve_status = async_to_raw_response_wrapper(
            phone_number_assignment_by_profile.retrieve_status,
        )


class PhoneNumberAssignmentByProfileResourceWithStreamingResponse:
    def __init__(self, phone_number_assignment_by_profile: PhoneNumberAssignmentByProfileResource) -> None:
        self._phone_number_assignment_by_profile = phone_number_assignment_by_profile

        self.assign = to_streamed_response_wrapper(
            phone_number_assignment_by_profile.assign,
        )
        self.retrieve_phone_number_status = to_streamed_response_wrapper(
            phone_number_assignment_by_profile.retrieve_phone_number_status,
        )
        self.retrieve_status = to_streamed_response_wrapper(
            phone_number_assignment_by_profile.retrieve_status,
        )


class AsyncPhoneNumberAssignmentByProfileResourceWithStreamingResponse:
    def __init__(self, phone_number_assignment_by_profile: AsyncPhoneNumberAssignmentByProfileResource) -> None:
        self._phone_number_assignment_by_profile = phone_number_assignment_by_profile

        self.assign = async_to_streamed_response_wrapper(
            phone_number_assignment_by_profile.assign,
        )
        self.retrieve_phone_number_status = async_to_streamed_response_wrapper(
            phone_number_assignment_by_profile.retrieve_phone_number_status,
        )
        self.retrieve_status = async_to_streamed_response_wrapper(
            phone_number_assignment_by_profile.retrieve_status,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .bucket_list_response import BucketListResponse as BucketListResponse
from .bucket_retrieve_response import BucketRetrieveResponse as BucketRetrieveResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["WirelessRetrieveRegionsResponse", "Data"]


class Data(BaseModel):
    code: str
    """The unique code of the region."""

    name: str
    """The name of the region."""

    inserted_at: Optional[datetime] = None
    """Timestamp when the region was inserted."""

    updated_at: Optional[datetime] = None
    """Timestamp when the region was last updated."""


class WirelessRetrieveRegionsResponse(BaseModel):
    data: Optional[List[Data]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from .buckets.buckets import (
    BucketsResource,
    AsyncBucketsResource,
    BucketsResourceWithRawResponse,
    AsyncBucketsResourceWithRawResponse,
    BucketsResourceWithStreamingResponse,
    AsyncBucketsResourceWithStreamingResponse,
)
from .migration_sources import (
    MigrationSourcesResource,
    AsyncMigrationSourcesResource,
    MigrationSourcesResourceWithRawResponse,
    AsyncMigrationSourcesResourceWithRawResponse,
    MigrationSourcesResourceWithStreamingResponse,
    AsyncMigrationSourcesResourceWithStreamingResponse,
)
from .migrations.migrations import (
    MigrationsResource,
    AsyncMigrationsResource,
    MigrationsResourceWithRawResponse,
    AsyncMigrationsResourceWithRawResponse,
    MigrationsResourceWithStreamingResponse,
    AsyncMigrationsResourceWithStreamingResponse,
)
from ...types.storage_list_migration_source_coverage_response import StorageListMigrationSourceCoverageResponse

__all__ = ["StorageResource", "AsyncStorageResource"]


class StorageResource(SyncAPIResource):
    @cached_property
    def buckets(self) -> BucketsResource:
        return BucketsResource(self._client)

    @cached_property
    def migration_sources(self) -> MigrationSourcesResource:
        return MigrationSourcesResource(self._client)

    @cached_property
    def migrations(self) -> MigrationsResource:
        return MigrationsResource(self._client)

    @cached_property
    def with_raw_response(self) -> StorageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return StorageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> StorageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return StorageResourceWithStreamingResponse(self)

    def list_migration_source_coverage(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StorageListMigrationSourceCoverageResponse:
        """List Migration Source coverage"""
        return self._get(
            "/storage/migration_source_coverage",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StorageListMigrationSourceCoverageResponse,
        )


class AsyncStorageResource(AsyncAPIResource):
    @cached_property
    def buckets(self) -> AsyncBucketsResource:
        return AsyncBucketsResource(self._client)

    @cached_property
    def migration_sources(self) -> AsyncMigrationSourcesResource:
        return AsyncMigrationSourcesResource(self._client)

    @cached_property
    def migrations(self) -> AsyncMigrationsResource:
        return AsyncMigrationsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncStorageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncStorageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncStorageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncStorageResourceWithStreamingResponse(self)

    async def list_migration_source_coverage(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StorageListMigrationSourceCoverageResponse:
        """List Migration Source coverage"""
        return await self._get(
            "/storage/migration_source_coverage",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StorageListMigrationSourceCoverageResponse,
        )


class StorageResourceWithRawResponse:
    def __init__(self, storage: StorageResource) -> None:
        self._storage = storage

        self.list_migration_source_coverage = to_raw_response_wrapper(
            storage.list_migration_source_coverage,
        )

    @cached_property
    def buckets(self) -> BucketsResourceWithRawResponse:
        return BucketsResourceWithRawResponse(self._storage.buckets)

    @cached_property
    def migration_sources(self) -> MigrationSourcesResourceWithRawResponse:
        return MigrationSourcesResourceWithRawResponse(self._storage.migration_sources)

    @cached_property
    def migrations(self) -> MigrationsResourceWithRawResponse:
        return MigrationsResourceWithRawResponse(self._storage.migrations)


class AsyncStorageResourceWithRawResponse:
    def __init__(self, storage: AsyncStorageResource) -> None:
        self._storage = storage

        self.list_migration_source_coverage = async_to_raw_response_wrapper(
            storage.list_migration_source_coverage,
        )

    @cached_property
    def buckets(self) -> AsyncBucketsResourceWithRawResponse:
        return AsyncBucketsResourceWithRawResponse(self._storage.buckets)

    @cached_property
    def migration_sources(self) -> AsyncMigrationSourcesResourceWithRawResponse:
        return AsyncMigrationSourcesResourceWithRawResponse(self._storage.migration_sources)

    @cached_property
    def migrations(self) -> AsyncMigrationsResourceWithRawResponse:
        return AsyncMigrationsResourceWithRawResponse(self._storage.migrations)


class StorageResourceWithStreamingResponse:
    def __init__(self, storage: StorageResource) -> None:
        self._storage = storage

        self.list_migration_source_coverage = to_streamed_response_wrapper(
            storage.list_migration_source_coverage,
        )

    @cached_property
    def buckets(self) -> BucketsResourceWithStreamingResponse:
        return BucketsResourceWithStreamingResponse(self._storage.buckets)

    @cached_property
    def migration_sources(self) -> MigrationSourcesResourceWithStreamingResponse:
        return MigrationSourcesResourceWithStreamingResponse(self._storage.migration_sources)

    @cached_property
    def migrations(self) -> MigrationsResourceWithStreamingResponse:
        return MigrationsResourceWithStreamingResponse(self._storage.migrations)


class AsyncStorageResourceWithStreamingResponse:
    def __init__(self, storage: AsyncStorageResource) -> None:
        self._storage = storage

        self.list_migration_source_coverage = async_to_streamed_response_wrapper(
            storage.list_migration_source_coverage,
        )

    @cached_property
    def buckets(self) -> AsyncBucketsResourceWithStreamingResponse:
        return AsyncBucketsResourceWithStreamingResponse(self._storage.buckets)

    @cached_property
    def migration_sources(self) -> AsyncMigrationSourcesResourceWithStreamingResponse:
        return AsyncMigrationSourcesResourceWithStreamingResponse(self._storage.migration_sources)

    @cached_property
    def migrations(self) -> AsyncMigrationsResourceWithStreamingResponse:
        return AsyncMigrationsResourceWithStreamingResponse(self._storage.migrations)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["SimCardOrderListParams", "Filter", "FilterAddress", "FilterCost", "Page"]


class SimCardOrderListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter for SIM card orders (deepObject style).

    Originally: filter[created_at], filter[updated_at], filter[quantity],
    filter[cost.amount], filter[cost.currency], filter[address.id],
    filter[address.street_address], filter[address.extended_address],
    filter[address.locality], filter[address.administrative_area],
    filter[address.country_code], filter[address.postal_code]
    """

    page: Page
    """Consolidated pagination parameter (deepObject style).

    Originally: page[number], page[size]
    """


class FilterAddress(TypedDict, total=False):
    id: str
    """Uniquely identifies the address for the order."""

    administrative_area: str
    """Filter by state or province where the address is located."""

    country_code: str
    """
    Filter by the mobile operator two-character (ISO 3166-1 alpha-2) origin country
    code.
    """

    extended_address: str
    """
    Returns entries with matching name of the supplemental field for address
    information.
    """

    locality: str
    """Filter by the name of the city where the address is located."""

    postal_code: str
    """Filter by postal code for the address."""

    street_address: str
    """Returns entries with matching name of the street where the address is located."""


class FilterCost(TypedDict, total=False):
    amount: str
    """The total monetary amount of the order."""

    currency: str
    """Filter by ISO 4217 currency string."""


class Filter(TypedDict, total=False):
    address: FilterAddress

    cost: FilterCost

    created_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """
    Filter by ISO 8601 formatted date-time string matching resource creation
    date-time.
    """

    quantity: int
    """Filter orders by how many SIM cards were ordered."""

    updated_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """
    Filter by ISO 8601 formatted date-time string matching resource last update
    date-time.
    """


class Page(TypedDict, total=False):
    number: int
    """The page number to load."""

    size: int
    """The size of the page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...types import address_list_params, address_create_params
from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.address_list_response import AddressListResponse
from ...types.address_create_response import AddressCreateResponse
from ...types.address_delete_response import AddressDeleteResponse
from ...types.address_retrieve_response import AddressRetrieveResponse

__all__ = ["AddressesResource", "AsyncAddressesResource"]


class AddressesResource(SyncAPIResource):
    @cached_property
    def actions(self) -> ActionsResource:
        return ActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AddressesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AddressesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AddressesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AddressesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        business_name: str,
        country_code: str,
        first_name: str,
        last_name: str,
        locality: str,
        street_address: str,
        address_book: bool | Omit = omit,
        administrative_area: str | Omit = omit,
        borough: str | Omit = omit,
        customer_reference: str | Omit = omit,
        extended_address: str | Omit = omit,
        neighborhood: str | Omit = omit,
        phone_number: str | Omit = omit,
        postal_code: str | Omit = omit,
        validate_address: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AddressCreateResponse:
        """
        Creates an address.

        Args:
          business_name: The business name associated with the address. An address must have either a
              first last name or a business name.

          country_code: The two-character (ISO 3166-1 alpha-2) country code of the address.

          first_name: The first name associated with the address. An address must have either a first
              last name or a business name.

          last_name: The last name associated with the address. An address must have either a first
              last name or a business name.

          locality: The locality of the address. For US addresses, this corresponds to the city of
              the address.

          street_address: The primary street address information about the address.

          address_book: Indicates whether or not the address should be considered part of your list of
              addresses that appear for regular use.

          administrative_area: The locality of the address. For US addresses, this corresponds to the state of
              the address.

          borough: The borough of the address. This field is not used for addresses in the US but
              is used for some international addresses.

          customer_reference: A customer reference string for customer look ups.

          extended_address: Additional street address information about the address such as, but not limited
              to, unit number or apartment number.

          neighborhood: The neighborhood of the address. This field is not used for addresses in the US
              but is used for some international addresses.

          phone_number: The phone number associated with the address.

          postal_code: The postal code of the address.

          validate_address: Indicates whether or not the address should be validated for emergency use upon
              creation or not. This should be left with the default value of `true` unless you
              have used the `/addresses/actions/validate` endpoint to validate the address
              separately prior to creation. If an address is not validated for emergency use
              upon creation and it is not valid, it will not be able to be used for emergency
              services.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/addresses",
            body=maybe_transform(
                {
                    "business_name": business_name,
                    "country_code": country_code,
                    "first_name": first_name,
                    "last_name": last_name,
                    "locality": locality,
                    "street_address": street_address,
                    "address_book": address_book,
                    "administrative_area": administrative_area,
                    "borough": borough,
                    "customer_reference": customer_reference,
                    "extended_address": extended_address,
                    "neighborhood": neighborhood,
                    "phone_number": phone_number,
                    "postal_code": postal_code,
                    "validate_address": validate_address,
                },
                address_create_params.AddressCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AddressCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AddressRetrieveResponse:
        """
        Retrieves the details of an existing address.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/addresses/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AddressRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: address_list_params.Filter | Omit = omit,
        page: address_list_params.Page | Omit = omit,
        sort: Literal["created_at", "first_name", "last_name", "business_name", "street_address"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AddressListResponse:
        """
        Returns a list of your addresses.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[customer_reference][eq], filter[customer_reference][contains],
              filter[used_as_emergency], filter[street_address][contains],
              filter[address_book][eq]

          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          sort: Specifies the sort order for results. By default sorting direction is ascending.
              To have the results sorted in descending order add the <code> -</code>
              prefix.<br/><br/> That is: <ul>

                <li>
                  <code>street_address</code>: sorts the result by the
                  <code>street_address</code> field in ascending order.
                </li>

                <li>
                  <code>-street_address</code>: sorts the result by the
                  <code>street_address</code> field in descending order.
                </li>
              </ul> <br/> If not given, results are sorted by <code>created_at</code> in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/addresses",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                        "sort": sort,
                    },
                    address_list_params.AddressListParams,
                ),
            ),
            cast_to=AddressListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AddressDeleteResponse:
        """
        Deletes an existing address.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/addresses/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AddressDeleteResponse,
        )


class AsyncAddressesResource(AsyncAPIResource):
    @cached_property
    def actions(self) -> AsyncActionsResource:
        return AsyncActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAddressesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAddressesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAddressesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncAddressesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        business_name: str,
        country_code: str,
        first_name: str,
        last_name: str,
        locality: str,
        street_address: str,
        address_book: bool | Omit = omit,
        administrative_area: str | Omit = omit,
        borough: str | Omit = omit,
        customer_reference: str | Omit = omit,
        extended_address: str | Omit = omit,
        neighborhood: str | Omit = omit,
        phone_number: str | Omit = omit,
        postal_code: str | Omit = omit,
        validate_address: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AddressCreateResponse:
        """
        Creates an address.

        Args:
          business_name: The business name associated with the address. An address must have either a
              first last name or a business name.

          country_code: The two-character (ISO 3166-1 alpha-2) country code of the address.

          first_name: The first name associated with the address. An address must have either a first
              last name or a business name.

          last_name: The last name associated with the address. An address must have either a first
              last name or a business name.

          locality: The locality of the address. For US addresses, this corresponds to the city of
              the address.

          street_address: The primary street address information about the address.

          address_book: Indicates whether or not the address should be considered part of your list of
              addresses that appear for regular use.

          administrative_area: The locality of the address. For US addresses, this corresponds to the state of
              the address.

          borough: The borough of the address. This field is not used for addresses in the US but
              is used for some international addresses.

          customer_reference: A customer reference string for customer look ups.

          extended_address: Additional street address information about the address such as, but not limited
              to, unit number or apartment number.

          neighborhood: The neighborhood of the address. This field is not used for addresses in the US
              but is used for some international addresses.

          phone_number: The phone number associated with the address.

          postal_code: The postal code of the address.

          validate_address: Indicates whether or not the address should be validated for emergency use upon
              creation or not. This should be left with the default value of `true` unless you
              have used the `/addresses/actions/validate` endpoint to validate the address
              separately prior to creation. If an address is not validated for emergency use
              upon creation and it is not valid, it will not be able to be used for emergency
              services.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/addresses",
            body=await async_maybe_transform(
                {
                    "business_name": business_name,
                    "country_code": country_code,
                    "first_name": first_name,
                    "last_name": last_name,
                    "locality": locality,
                    "street_address": street_address,
                    "address_book": address_book,
                    "administrative_area": administrative_area,
                    "borough": borough,
                    "customer_reference": customer_reference,
                    "extended_address": extended_address,
                    "neighborhood": neighborhood,
                    "phone_number": phone_number,
                    "postal_code": postal_code,
                    "validate_address": validate_address,
                },
                address_create_params.AddressCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AddressCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AddressRetrieveResponse:
        """
        Retrieves the details of an existing address.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/addresses/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AddressRetrieveResponse,
        )

    async def list(
        self,
        *,
        filter: address_list_params.Filter | Omit = omit,
        page: address_list_params.Page | Omit = omit,
        sort: Literal["created_at", "first_name", "last_name", "business_name", "street_address"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AddressListResponse:
        """
        Returns a list of your addresses.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[customer_reference][eq], filter[customer_reference][contains],
              filter[used_as_emergency], filter[street_address][contains],
              filter[address_book][eq]

          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          sort: Specifies the sort order for results. By default sorting direction is ascending.
              To have the results sorted in descending order add the <code> -</code>
              prefix.<br/><br/> That is: <ul>

                <li>
                  <code>street_address</code>: sorts the result by the
                  <code>street_address</code> field in ascending order.
                </li>

                <li>
                  <code>-street_address</code>: sorts the result by the
                  <code>street_address</code> field in descending order.
                </li>
              </ul> <br/> If not given, results are sorted by <code>created_at</code> in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/addresses",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                        "sort": sort,
                    },
                    address_list_params.AddressListParams,
                ),
            ),
            cast_to=AddressListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AddressDeleteResponse:
        """
        Deletes an existing address.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/addresses/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AddressDeleteResponse,
        )


class AddressesResourceWithRawResponse:
    def __init__(self, addresses: AddressesResource) -> None:
        self._addresses = addresses

        self.create = to_raw_response_wrapper(
            addresses.create,
        )
        self.retrieve = to_raw_response_wrapper(
            addresses.retrieve,
        )
        self.list = to_raw_response_wrapper(
            addresses.list,
        )
        self.delete = to_raw_response_wrapper(
            addresses.delete,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithRawResponse:
        return ActionsResourceWithRawResponse(self._addresses.actions)


class AsyncAddressesResourceWithRawResponse:
    def __init__(self, addresses: AsyncAddressesResource) -> None:
        self._addresses = addresses

        self.create = async_to_raw_response_wrapper(
            addresses.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            addresses.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            addresses.list,
        )
        self.delete = async_to_raw_response_wrapper(
            addresses.delete,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithRawResponse:
        return AsyncActionsResourceWithRawResponse(self._addresses.actions)


class AddressesResourceWithStreamingResponse:
    def __init__(self, addresses: AddressesResource) -> None:
        self._addresses = addresses

        self.create = to_streamed_response_wrapper(
            addresses.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            addresses.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            addresses.list,
        )
        self.delete = to_streamed_response_wrapper(
            addresses.delete,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithStreamingResponse:
        return ActionsResourceWithStreamingResponse(self._addresses.actions)


class AsyncAddressesResourceWithStreamingResponse:
    def __init__(self, addresses: AsyncAddressesResource) -> None:
        self._addresses = addresses

        self.create = async_to_streamed_response_wrapper(
            addresses.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            addresses.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            addresses.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            addresses.delete,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithStreamingResponse:
        return AsyncActionsResourceWithStreamingResponse(self._addresses.actions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .phone_number_campaign import PhoneNumberCampaign

__all__ = ["PhoneNumberCampaignListResponse"]


class PhoneNumberCampaignListResponse(BaseModel):
    page: int

    records: List[PhoneNumberCampaign]

    total_records: int = FieldInfo(alias="totalRecords")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from .jobs import (
    JobsResource,
    AsyncJobsResource,
    JobsResourceWithRawResponse,
    AsyncJobsResourceWithRawResponse,
    JobsResourceWithStreamingResponse,
    AsyncJobsResourceWithStreamingResponse,
)
from .voice import (
    VoiceResource,
    AsyncVoiceResource,
    VoiceResourceWithRawResponse,
    AsyncVoiceResourceWithRawResponse,
    VoiceResourceWithStreamingResponse,
    AsyncVoiceResourceWithStreamingResponse,
)
from ...types import phone_number_list_params, phone_number_update_params, phone_number_slim_list_params
from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from .messaging import (
    MessagingResource,
    AsyncMessagingResource,
    MessagingResourceWithRawResponse,
    AsyncMessagingResourceWithRawResponse,
    MessagingResourceWithStreamingResponse,
    AsyncMessagingResourceWithStreamingResponse,
)
from .voicemail import (
    VoicemailResource,
    AsyncVoicemailResource,
    VoicemailResourceWithRawResponse,
    AsyncVoicemailResourceWithRawResponse,
    VoicemailResourceWithStreamingResponse,
    AsyncVoicemailResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .csv_downloads import (
    CsvDownloadsResource,
    AsyncCsvDownloadsResource,
    CsvDownloadsResourceWithRawResponse,
    AsyncCsvDownloadsResourceWithRawResponse,
    CsvDownloadsResourceWithStreamingResponse,
    AsyncCsvDownloadsResourceWithStreamingResponse,
)
from ..._base_client import make_request_options
from ...types.phone_number_list_response import PhoneNumberListResponse
from ...types.phone_number_delete_response import PhoneNumberDeleteResponse
from ...types.phone_number_update_response import PhoneNumberUpdateResponse
from ...types.phone_number_retrieve_response import PhoneNumberRetrieveResponse
from ...types.phone_number_slim_list_response import PhoneNumberSlimListResponse

__all__ = ["PhoneNumbersResource", "AsyncPhoneNumbersResource"]


class PhoneNumbersResource(SyncAPIResource):
    @cached_property
    def actions(self) -> ActionsResource:
        return ActionsResource(self._client)

    @cached_property
    def csv_downloads(self) -> CsvDownloadsResource:
        return CsvDownloadsResource(self._client)

    @cached_property
    def jobs(self) -> JobsResource:
        return JobsResource(self._client)

    @cached_property
    def messaging(self) -> MessagingResource:
        return MessagingResource(self._client)

    @cached_property
    def voice(self) -> VoiceResource:
        return VoiceResource(self._client)

    @cached_property
    def voicemail(self) -> VoicemailResource:
        return VoicemailResource(self._client)

    @cached_property
    def with_raw_response(self) -> PhoneNumbersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return PhoneNumbersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PhoneNumbersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return PhoneNumbersResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberRetrieveResponse:
        """
        Retrieve a phone number

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/phone_numbers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        billing_group_id: str | Omit = omit,
        connection_id: str | Omit = omit,
        customer_reference: str | Omit = omit,
        external_pin: str | Omit = omit,
        hd_voice_enabled: bool | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberUpdateResponse:
        """
        Update a phone number

        Args:
          billing_group_id: Identifies the billing group associated with the phone number.

          connection_id: Identifies the connection associated with the phone number.

          customer_reference: A customer reference string for customer look ups.

          external_pin: If someone attempts to port your phone number away from Telnyx and your phone
              number has an external PIN set, we will attempt to verify that you provided the
              correct external PIN to the winning carrier. Note that not all carriers
              cooperate with this security mechanism.

          hd_voice_enabled: Indicates whether HD voice is enabled for this number.

          tags: A list of user-assigned tags to help organize phone numbers.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/phone_numbers/{id}",
            body=maybe_transform(
                {
                    "billing_group_id": billing_group_id,
                    "connection_id": connection_id,
                    "customer_reference": customer_reference,
                    "external_pin": external_pin,
                    "hd_voice_enabled": hd_voice_enabled,
                    "tags": tags,
                },
                phone_number_update_params.PhoneNumberUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberUpdateResponse,
        )

    def list(
        self,
        *,
        filter: phone_number_list_params.Filter | Omit = omit,
        page: phone_number_list_params.Page | Omit = omit,
        sort: Literal["purchased_at", "phone_number", "connection_name", "usage_payment_method"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberListResponse:
        """
        List phone numbers

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[tag],
              filter[phone_number], filter[status], filter[country_iso_alpha2],
              filter[connection_id], filter[voice.connection_name],
              filter[voice.usage_payment_method], filter[billing_group_id],
              filter[emergency_address_id], filter[customer_reference], filter[number_type],
              filter[source]

          page: Consolidated page parameter (deepObject style). Originally: page[size],
              page[number]

          sort: Specifies the sort order for results. If not given, results are sorted by
              created_at in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/phone_numbers",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                        "sort": sort,
                    },
                    phone_number_list_params.PhoneNumberListParams,
                ),
            ),
            cast_to=PhoneNumberListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberDeleteResponse:
        """
        Delete a phone number

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/phone_numbers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberDeleteResponse,
        )

    def slim_list(
        self,
        *,
        filter: phone_number_slim_list_params.Filter | Omit = omit,
        include_connection: bool | Omit = omit,
        include_tags: bool | Omit = omit,
        page: phone_number_slim_list_params.Page | Omit = omit,
        sort: Literal["purchased_at", "phone_number", "connection_name", "usage_payment_method"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberSlimListResponse:
        """
        List phone numbers, This endpoint is a lighter version of the /phone_numbers
        endpoint having higher performance and rate limit.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[tag],
              filter[phone_number], filter[status], filter[country_iso_alpha2],
              filter[connection_id], filter[voice.connection_name],
              filter[voice.usage_payment_method], filter[billing_group_id],
              filter[emergency_address_id], filter[customer_reference], filter[number_type],
              filter[source]

          include_connection: Include the connection associated with the phone number.

          include_tags: Include the tags associated with the phone number.

          page: Consolidated page parameter (deepObject style). Originally: page[size],
              page[number]

          sort: Specifies the sort order for results. If not given, results are sorted by
              created_at in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/phone_numbers/slim",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "include_connection": include_connection,
                        "include_tags": include_tags,
                        "page": page,
                        "sort": sort,
                    },
                    phone_number_slim_list_params.PhoneNumberSlimListParams,
                ),
            ),
            cast_to=PhoneNumberSlimListResponse,
        )


class AsyncPhoneNumbersResource(AsyncAPIResource):
    @cached_property
    def actions(self) -> AsyncActionsResource:
        return AsyncActionsResource(self._client)

    @cached_property
    def csv_downloads(self) -> AsyncCsvDownloadsResource:
        return AsyncCsvDownloadsResource(self._client)

    @cached_property
    def jobs(self) -> AsyncJobsResource:
        return AsyncJobsResource(self._client)

    @cached_property
    def messaging(self) -> AsyncMessagingResource:
        return AsyncMessagingResource(self._client)

    @cached_property
    def voice(self) -> AsyncVoiceResource:
        return AsyncVoiceResource(self._client)

    @cached_property
    def voicemail(self) -> AsyncVoicemailResource:
        return AsyncVoicemailResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncPhoneNumbersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPhoneNumbersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPhoneNumbersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncPhoneNumbersResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberRetrieveResponse:
        """
        Retrieve a phone number

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/phone_numbers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        billing_group_id: str | Omit = omit,
        connection_id: str | Omit = omit,
        customer_reference: str | Omit = omit,
        external_pin: str | Omit = omit,
        hd_voice_enabled: bool | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberUpdateResponse:
        """
        Update a phone number

        Args:
          billing_group_id: Identifies the billing group associated with the phone number.

          connection_id: Identifies the connection associated with the phone number.

          customer_reference: A customer reference string for customer look ups.

          external_pin: If someone attempts to port your phone number away from Telnyx and your phone
              number has an external PIN set, we will attempt to verify that you provided the
              correct external PIN to the winning carrier. Note that not all carriers
              cooperate with this security mechanism.

          hd_voice_enabled: Indicates whether HD voice is enabled for this number.

          tags: A list of user-assigned tags to help organize phone numbers.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/phone_numbers/{id}",
            body=await async_maybe_transform(
                {
                    "billing_group_id": billing_group_id,
                    "connection_id": connection_id,
                    "customer_reference": customer_reference,
                    "external_pin": external_pin,
                    "hd_voice_enabled": hd_voice_enabled,
                    "tags": tags,
                },
                phone_number_update_params.PhoneNumberUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberUpdateResponse,
        )

    async def list(
        self,
        *,
        filter: phone_number_list_params.Filter | Omit = omit,
        page: phone_number_list_params.Page | Omit = omit,
        sort: Literal["purchased_at", "phone_number", "connection_name", "usage_payment_method"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberListResponse:
        """
        List phone numbers

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[tag],
              filter[phone_number], filter[status], filter[country_iso_alpha2],
              filter[connection_id], filter[voice.connection_name],
              filter[voice.usage_payment_method], filter[billing_group_id],
              filter[emergency_address_id], filter[customer_reference], filter[number_type],
              filter[source]

          page: Consolidated page parameter (deepObject style). Originally: page[size],
              page[number]

          sort: Specifies the sort order for results. If not given, results are sorted by
              created_at in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/phone_numbers",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                        "sort": sort,
                    },
                    phone_number_list_params.PhoneNumberListParams,
                ),
            ),
            cast_to=PhoneNumberListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberDeleteResponse:
        """
        Delete a phone number

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/phone_numbers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberDeleteResponse,
        )

    async def slim_list(
        self,
        *,
        filter: phone_number_slim_list_params.Filter | Omit = omit,
        include_connection: bool | Omit = omit,
        include_tags: bool | Omit = omit,
        page: phone_number_slim_list_params.Page | Omit = omit,
        sort: Literal["purchased_at", "phone_number", "connection_name", "usage_payment_method"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberSlimListResponse:
        """
        List phone numbers, This endpoint is a lighter version of the /phone_numbers
        endpoint having higher performance and rate limit.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[tag],
              filter[phone_number], filter[status], filter[country_iso_alpha2],
              filter[connection_id], filter[voice.connection_name],
              filter[voice.usage_payment_method], filter[billing_group_id],
              filter[emergency_address_id], filter[customer_reference], filter[number_type],
              filter[source]

          include_connection: Include the connection associated with the phone number.

          include_tags: Include the tags associated with the phone number.

          page: Consolidated page parameter (deepObject style). Originally: page[size],
              page[number]

          sort: Specifies the sort order for results. If not given, results are sorted by
              created_at in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/phone_numbers/slim",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter": filter,
                        "include_connection": include_connection,
                        "include_tags": include_tags,
                        "page": page,
                        "sort": sort,
                    },
                    phone_number_slim_list_params.PhoneNumberSlimListParams,
                ),
            ),
            cast_to=PhoneNumberSlimListResponse,
        )


class PhoneNumbersResourceWithRawResponse:
    def __init__(self, phone_numbers: PhoneNumbersResource) -> None:
        self._phone_numbers = phone_numbers

        self.retrieve = to_raw_response_wrapper(
            phone_numbers.retrieve,
        )
        self.update = to_raw_response_wrapper(
            phone_numbers.update,
        )
        self.list = to_raw_response_wrapper(
            phone_numbers.list,
        )
        self.delete = to_raw_response_wrapper(
            phone_numbers.delete,
        )
        self.slim_list = to_raw_response_wrapper(
            phone_numbers.slim_list,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithRawResponse:
        return ActionsResourceWithRawResponse(self._phone_numbers.actions)

    @cached_property
    def csv_downloads(self) -> CsvDownloadsResourceWithRawResponse:
        return CsvDownloadsResourceWithRawResponse(self._phone_numbers.csv_downloads)

    @cached_property
    def jobs(self) -> JobsResourceWithRawResponse:
        return JobsResourceWithRawResponse(self._phone_numbers.jobs)

    @cached_property
    def messaging(self) -> MessagingResourceWithRawResponse:
        return MessagingResourceWithRawResponse(self._phone_numbers.messaging)

    @cached_property
    def voice(self) -> VoiceResourceWithRawResponse:
        return VoiceResourceWithRawResponse(self._phone_numbers.voice)

    @cached_property
    def voicemail(self) -> VoicemailResourceWithRawResponse:
        return VoicemailResourceWithRawResponse(self._phone_numbers.voicemail)


class AsyncPhoneNumbersResourceWithRawResponse:
    def __init__(self, phone_numbers: AsyncPhoneNumbersResource) -> None:
        self._phone_numbers = phone_numbers

        self.retrieve = async_to_raw_response_wrapper(
            phone_numbers.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            phone_numbers.update,
        )
        self.list = async_to_raw_response_wrapper(
            phone_numbers.list,
        )
        self.delete = async_to_raw_response_wrapper(
            phone_numbers.delete,
        )
        self.slim_list = async_to_raw_response_wrapper(
            phone_numbers.slim_list,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithRawResponse:
        return AsyncActionsResourceWithRawResponse(self._phone_numbers.actions)

    @cached_property
    def csv_downloads(self) -> AsyncCsvDownloadsResourceWithRawResponse:
        return AsyncCsvDownloadsResourceWithRawResponse(self._phone_numbers.csv_downloads)

    @cached_property
    def jobs(self) -> AsyncJobsResourceWithRawResponse:
        return AsyncJobsResourceWithRawResponse(self._phone_numbers.jobs)

    @cached_property
    def messaging(self) -> AsyncMessagingResourceWithRawResponse:
        return AsyncMessagingResourceWithRawResponse(self._phone_numbers.messaging)

    @cached_property
    def voice(self) -> AsyncVoiceResourceWithRawResponse:
        return AsyncVoiceResourceWithRawResponse(self._phone_numbers.voice)

    @cached_property
    def voicemail(self) -> AsyncVoicemailResourceWithRawResponse:
        return AsyncVoicemailResourceWithRawResponse(self._phone_numbers.voicemail)


class PhoneNumbersResourceWithStreamingResponse:
    def __init__(self, phone_numbers: PhoneNumbersResource) -> None:
        self._phone_numbers = phone_numbers

        self.retrieve = to_streamed_response_wrapper(
            phone_numbers.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            phone_numbers.update,
        )
        self.list = to_streamed_response_wrapper(
            phone_numbers.list,
        )
        self.delete = to_streamed_response_wrapper(
            phone_numbers.delete,
        )
        self.slim_list = to_streamed_response_wrapper(
            phone_numbers.slim_list,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithStreamingResponse:
        return ActionsResourceWithStreamingResponse(self._phone_numbers.actions)

    @cached_property
    def csv_downloads(self) -> CsvDownloadsResourceWithStreamingResponse:
        return CsvDownloadsResourceWithStreamingResponse(self._phone_numbers.csv_downloads)

    @cached_property
    def jobs(self) -> JobsResourceWithStreamingResponse:
        return JobsResourceWithStreamingResponse(self._phone_numbers.jobs)

    @cached_property
    def messaging(self) -> MessagingResourceWithStreamingResponse:
        return MessagingResourceWithStreamingResponse(self._phone_numbers.messaging)

    @cached_property
    def voice(self) -> VoiceResourceWithStreamingResponse:
        return VoiceResourceWithStreamingResponse(self._phone_numbers.voice)

    @cached_property
    def voicemail(self) -> VoicemailResourceWithStreamingResponse:
        return VoicemailResourceWithStreamingResponse(self._phone_numbers.voicemail)


class AsyncPhoneNumbersResourceWithStreamingResponse:
    def __init__(self, phone_numbers: AsyncPhoneNumbersResource) -> None:
        self._phone_numbers = phone_numbers

        self.retrieve = async_to_streamed_response_wrapper(
            phone_numbers.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            phone_numbers.update,
        )
        self.list = async_to_streamed_response_wrapper(
            phone_numbers.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            phone_numbers.delete,
        )
        self.slim_list = async_to_streamed_response_wrapper(
            phone_numbers.slim_list,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithStreamingResponse:
        return AsyncActionsResourceWithStreamingResponse(self._phone_numbers.actions)

    @cached_property
    def csv_downloads(self) -> AsyncCsvDownloadsResourceWithStreamingResponse:
        return AsyncCsvDownloadsResourceWithStreamingResponse(self._phone_numbers.csv_downloads)

    @cached_property
    def jobs(self) -> AsyncJobsResourceWithStreamingResponse:
        return AsyncJobsResourceWithStreamingResponse(self._phone_numbers.jobs)

    @cached_property
    def messaging(self) -> AsyncMessagingResourceWithStreamingResponse:
        return AsyncMessagingResourceWithStreamingResponse(self._phone_numbers.messaging)

    @cached_property
    def voice(self) -> AsyncVoiceResourceWithStreamingResponse:
        return AsyncVoiceResourceWithStreamingResponse(self._phone_numbers.voice)

    @cached_property
    def voicemail(self) -> AsyncVoicemailResourceWithStreamingResponse:
        return AsyncVoicemailResourceWithStreamingResponse(self._phone_numbers.voicemail)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["PhoneNumberListParams", "Filter", "FilterNumberType", "FilterVoiceConnectionName", "Page"]


class PhoneNumberListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[tag], filter[phone_number], filter[status],
    filter[country_iso_alpha2], filter[connection_id],
    filter[voice.connection_name], filter[voice.usage_payment_method],
    filter[billing_group_id], filter[emergency_address_id],
    filter[customer_reference], filter[number_type], filter[source]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[size], page[number]
    """

    sort: Literal["purchased_at", "phone_number", "connection_name", "usage_payment_method"]
    """Specifies the sort order for results.

    If not given, results are sorted by created_at in descending order.
    """


class FilterNumberType(TypedDict, total=False):
    eq: Literal["local", "national", "toll_free", "mobile", "shared_cost"]
    """Filter phone numbers by phone number type."""


class FilterVoiceConnectionName(TypedDict, total=False):
    contains: str
    """Filter contains connection name. Requires at least three characters."""

    ends_with: str
    """Filter ends with connection name. Requires at least three characters."""

    eq: str
    """Filter by connection name."""

    starts_with: str
    """Filter starts with connection name. Requires at least three characters."""


class Filter(TypedDict, total=False):
    billing_group_id: str
    """Filter by the billing_group_id associated with phone numbers.

    To filter to only phone numbers that have no billing group associated them, set
    the value of this filter to the string 'null'.
    """

    connection_id: str
    """Filter by connection_id."""

    country_iso_alpha2: Union[str, SequenceNotStr[str]]
    """Filter by phone number country ISO alpha-2 code.

    Can be a single value or an array of values.
    """

    customer_reference: str
    """Filter numbers via the customer_reference set."""

    emergency_address_id: str
    """Filter by the emergency_address_id associated with phone numbers.

    To filter only phone numbers that have no emergency address associated with
    them, set the value of this filter to the string 'null'.
    """

    number_type: FilterNumberType
    """Filter phone numbers by phone number type."""

    phone_number: str
    """Filter by phone number.

    Requires at least three digits. Non-numerical characters will result in no
    values being returned.
    """

    source: Literal["ported", "purchased"]
    """Filter phone numbers by their source.

    Use 'ported' for numbers ported from other carriers, or 'purchased' for numbers
    bought directly from Telnyx.
    """

    status: Literal[
        "purchase-pending",
        "purchase-failed",
        "port-pending",
        "active",
        "deleted",
        "port-failed",
        "emergency-only",
        "ported-out",
        "port-out-pending",
    ]
    """Filter by phone number status."""

    tag: str
    """Filter by phone number tags."""

    voice_connection_name: Annotated[FilterVoiceConnectionName, PropertyInfo(alias="voice.connection_name")]
    """Filter by voice connection name pattern matching."""

    voice_usage_payment_method: Annotated[
        Literal["pay-per-minute", "channel"], PropertyInfo(alias="voice.usage_payment_method")
    ]
    """Filter by usage_payment_method."""

    without_tags: Literal["true", "false"]
    """When set to 'true', filters for phone numbers that do not have any tags applied.

    All other values are ignored.
    """


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["NotificationProfile"]


class NotificationProfile(BaseModel):
    id: Optional[str] = None
    """A UUID."""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    name: Optional[str] = None
    """A human readable name."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .migration_source_params import MigrationSourceParams
from .buckets.pagination_meta_simple import PaginationMetaSimple

__all__ = ["MigrationSourceListResponse"]


class MigrationSourceListResponse(BaseModel):
    data: Optional[List[MigrationSourceParams]] = None

    meta: Optional[PaginationMetaSimple] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .campaign_sharing_status import CampaignSharingStatus

__all__ = ["CampaignGetSharingStatusResponse"]


class CampaignGetSharingStatusResponse(BaseModel):
    shared_by_me: Optional[CampaignSharingStatus] = FieldInfo(alias="sharedByMe", default=None)

    shared_with_me: Optional[CampaignSharingStatus] = FieldInfo(alias="sharedWithMe", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["DynamicEmergencyEndpoint"]


class DynamicEmergencyEndpoint(BaseModel):
    callback_number: str

    caller_name: str

    dynamic_emergency_address_id: str
    """An id of a currently active dynamic emergency location."""

    id: Optional[str] = None

    created_at: Optional[str] = None
    """ISO 8601 formatted date of when the resource was created"""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    sip_from_id: Optional[str] = None

    status: Optional[Literal["pending", "activated", "rejected"]] = None
    """Status of dynamic emergency address"""

    updated_at: Optional[str] = None
    """ISO 8601 formatted date of when the resource was last updated"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    PhoneNumberCampaign,
    PhoneNumberCampaignListResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPhoneNumberCampaigns:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        phone_number_campaign = client.phone_number_campaigns.create(
            campaign_id="4b300178-131c-d902-d54e-72d90ba1620j",
            phone_number="+18005550199",
        )
        assert_matches_type(PhoneNumberCampaign, phone_number_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.phone_number_campaigns.with_raw_response.create(
            campaign_id="4b300178-131c-d902-d54e-72d90ba1620j",
            phone_number="+18005550199",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_campaign = response.parse()
        assert_matches_type(PhoneNumberCampaign, phone_number_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.phone_number_campaigns.with_streaming_response.create(
            campaign_id="4b300178-131c-d902-d54e-72d90ba1620j",
            phone_number="+18005550199",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_campaign = response.parse()
            assert_matches_type(PhoneNumberCampaign, phone_number_campaign, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        phone_number_campaign = client.phone_number_campaigns.retrieve(
            "phoneNumber",
        )
        assert_matches_type(PhoneNumberCampaign, phone_number_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.phone_number_campaigns.with_raw_response.retrieve(
            "phoneNumber",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_campaign = response.parse()
        assert_matches_type(PhoneNumberCampaign, phone_number_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.phone_number_campaigns.with_streaming_response.retrieve(
            "phoneNumber",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_campaign = response.parse()
            assert_matches_type(PhoneNumberCampaign, phone_number_campaign, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `phone_number` but received ''"):
            client.phone_number_campaigns.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        phone_number_campaign = client.phone_number_campaigns.update(
            path_phone_number="phoneNumber",
            campaign_id="4b300178-131c-d902-d54e-72d90ba1620j",
            body_phone_number="+18005550199",
        )
        assert_matches_type(PhoneNumberCampaign, phone_number_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.phone_number_campaigns.with_raw_response.update(
            path_phone_number="phoneNumber",
            campaign_id="4b300178-131c-d902-d54e-72d90ba1620j",
            body_phone_number="+18005550199",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_campaign = response.parse()
        assert_matches_type(PhoneNumberCampaign, phone_number_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.phone_number_campaigns.with_streaming_response.update(
            path_phone_number="phoneNumber",
            campaign_id="4b300178-131c-d902-d54e-72d90ba1620j",
            body_phone_number="+18005550199",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_campaign = response.parse()
            assert_matches_type(PhoneNumberCampaign, phone_number_campaign, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `path_phone_number` but received ''"):
            client.phone_number_campaigns.with_raw_response.update(
                path_phone_number="",
                campaign_id="4b300178-131c-d902-d54e-72d90ba1620j",
                body_phone_number="+18005550199",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        phone_number_campaign = client.phone_number_campaigns.list()
        assert_matches_type(PhoneNumberCampaignListResponse, phone_number_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        phone_number_campaign = client.phone_number_campaigns.list(
            filter={
                "tcr_brand_id": "BRANDID",
                "tcr_campaign_id": "CAMPID3",
                "telnyx_brand_id": "f3575e15-32ce-400e-a4c0-dd78800c20b0",
                "telnyx_campaign_id": "f3575e15-32ce-400e-a4c0-dd78800c20b0",
            },
            page=0,
            records_per_page=0,
            sort="assignmentStatus",
        )
        assert_matches_type(PhoneNumberCampaignListResponse, phone_number_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.phone_number_campaigns.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_campaign = response.parse()
        assert_matches_type(PhoneNumberCampaignListResponse, phone_number_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.phone_number_campaigns.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_campaign = response.parse()
            assert_matches_type(PhoneNumberCampaignListResponse, phone_number_campaign, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        phone_number_campaign = client.phone_number_campaigns.delete(
            "phoneNumber",
        )
        assert_matches_type(PhoneNumberCampaign, phone_number_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.phone_number_campaigns.with_raw_response.delete(
            "phoneNumber",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_campaign = response.parse()
        assert_matches_type(PhoneNumberCampaign, phone_number_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.phone_number_campaigns.with_streaming_response.delete(
            "phoneNumber",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_campaign = response.parse()
            assert_matches_type(PhoneNumberCampaign, phone_number_campaign, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `phone_number` but received ''"):
            client.phone_number_campaigns.with_raw_response.delete(
                "",
            )


class TestAsyncPhoneNumberCampaigns:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        phone_number_campaign = await async_client.phone_number_campaigns.create(
            campaign_id="4b300178-131c-d902-d54e-72d90ba1620j",
            phone_number="+18005550199",
        )
        assert_matches_type(PhoneNumberCampaign, phone_number_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.phone_number_campaigns.with_raw_response.create(
            campaign_id="4b300178-131c-d902-d54e-72d90ba1620j",
            phone_number="+18005550199",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_campaign = await response.parse()
        assert_matches_type(PhoneNumberCampaign, phone_number_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.phone_number_campaigns.with_streaming_response.create(
            campaign_id="4b300178-131c-d902-d54e-72d90ba1620j",
            phone_number="+18005550199",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_campaign = await response.parse()
            assert_matches_type(PhoneNumberCampaign, phone_number_campaign, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        phone_number_campaign = await async_client.phone_number_campaigns.retrieve(
            "phoneNumber",
        )
        assert_matches_type(PhoneNumberCampaign, phone_number_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.phone_number_campaigns.with_raw_response.retrieve(
            "phoneNumber",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_campaign = await response.parse()
        assert_matches_type(PhoneNumberCampaign, phone_number_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.phone_number_campaigns.with_streaming_response.retrieve(
            "phoneNumber",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_campaign = await response.parse()
            assert_matches_type(PhoneNumberCampaign, phone_number_campaign, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `phone_number` but received ''"):
            await async_client.phone_number_campaigns.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        phone_number_campaign = await async_client.phone_number_campaigns.update(
            path_phone_number="phoneNumber",
            campaign_id="4b300178-131c-d902-d54e-72d90ba1620j",
            body_phone_number="+18005550199",
        )
        assert_matches_type(PhoneNumberCampaign, phone_number_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.phone_number_campaigns.with_raw_response.update(
            path_phone_number="phoneNumber",
            campaign_id="4b300178-131c-d902-d54e-72d90ba1620j",
            body_phone_number="+18005550199",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_campaign = await response.parse()
        assert_matches_type(PhoneNumberCampaign, phone_number_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.phone_number_campaigns.with_streaming_response.update(
            path_phone_number="phoneNumber",
            campaign_id="4b300178-131c-d902-d54e-72d90ba1620j",
            body_phone_number="+18005550199",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_campaign = await response.parse()
            assert_matches_type(PhoneNumberCampaign, phone_number_campaign, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `path_phone_number` but received ''"):
            await async_client.phone_number_campaigns.with_raw_response.update(
                path_phone_number="",
                campaign_id="4b300178-131c-d902-d54e-72d90ba1620j",
                body_phone_number="+18005550199",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        phone_number_campaign = await async_client.phone_number_campaigns.list()
        assert_matches_type(PhoneNumberCampaignListResponse, phone_number_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        phone_number_campaign = await async_client.phone_number_campaigns.list(
            filter={
                "tcr_brand_id": "BRANDID",
                "tcr_campaign_id": "CAMPID3",
                "telnyx_brand_id": "f3575e15-32ce-400e-a4c0-dd78800c20b0",
                "telnyx_campaign_id": "f3575e15-32ce-400e-a4c0-dd78800c20b0",
            },
            page=0,
            records_per_page=0,
            sort="assignmentStatus",
        )
        assert_matches_type(PhoneNumberCampaignListResponse, phone_number_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.phone_number_campaigns.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_campaign = await response.parse()
        assert_matches_type(PhoneNumberCampaignListResponse, phone_number_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.phone_number_campaigns.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_campaign = await response.parse()
            assert_matches_type(PhoneNumberCampaignListResponse, phone_number_campaign, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        phone_number_campaign = await async_client.phone_number_campaigns.delete(
            "phoneNumber",
        )
        assert_matches_type(PhoneNumberCampaign, phone_number_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.phone_number_campaigns.with_raw_response.delete(
            "phoneNumber",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_campaign = await response.parse()
        assert_matches_type(PhoneNumberCampaign, phone_number_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.phone_number_campaigns.with_streaming_response.delete(
            "phoneNumber",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_campaign = await response.parse()
            assert_matches_type(PhoneNumberCampaign, phone_number_campaign, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `phone_number` but received ''"):
            await async_client.phone_number_campaigns.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..record import Record
from ..._models import BaseModel
from ..pagination_meta import PaginationMeta
from ..interface_status import InterfaceStatus

__all__ = ["DefaultGatewayCreateResponse", "Data"]


class Data(Record):
    network_id: Optional[str] = None
    """Network ID."""

    record_type: Optional[str] = None  # type: ignore
    """Identifies the type of the resource."""

    status: Optional[InterfaceStatus] = None
    """The current status of the interface deployment."""

    wireguard_peer_id: Optional[str] = None
    """Wireguard peer ID."""


class DefaultGatewayCreateResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[PaginationMeta] = None
